@@ -24,6 +24,7 @@ import (
 	"fmt"
 	"os"
 	"path/filepath"
+	"regexp"
 
 	"github.com/pkg/errors"
 	"github.com/symfony-cli/symfony-cli/git"
@@ -31,8 +32,8 @@ import (
 )
 
 var (
-	ErrProjectRootNotFoundNoGitRemote = goerr.New("project root not found, current directory not linked to a Platform.sh project")
-	ErrNoGitBranchMatching            = goerr.New("current git branch name doesn't match any Platform.sh environments")
+	ErrProjectRootNotFoundNoGitRemote = goerr.New("project root not found, current directory not linked to a Platform.sh/Upsun project")
+	ErrNoGitBranchMatching            = goerr.New("current git branch name doesn't match any Platform.sh/Upsun environments")
 )
 
 type Project struct {
@@ -42,13 +43,17 @@ type Project struct {
 }
 
 func ProjectFromDir(dir string, debug bool) (*Project, error) {
-	projectRoot, projectID := guessProjectRoot(dir, debug)
-	if projectID == "" {
-		return nil, errors.New("unable to get project root")
+	projectRoot := repositoryRootDir(dir)
+	if projectRoot == "" {
+		return nil, errors.New("unable to get project repository root")
 	}
-	envID, err := potentialCurrentEnvironmentID(projectRoot)
-	if err != nil {
-		return nil, errors.Wrap(err, "unable to get current env")
+	projectID := getProjectID(projectRoot, debug)
+	if projectID == "" {
+		return nil, errors.New("unable to get project id")
+	}
+	envID := git.GetCurrentBranch(projectRoot)
+	if envID == "" {
+		return nil, errors.New("unable to get current env: unable to retrieve the current Git branch name")
 	}
 	app := GuessSelectedAppByDirectory(dir, FindLocalApplications(projectRoot))
 	if app == nil {
@@ -67,21 +72,13 @@ func GetProjectRoot(debug bool) (string, error) {
 		return "", errors.WithStack(err)
 	}
 
-	if projectRoot, _ := guessProjectRoot(currentDir, debug); projectRoot != "" {
+	if projectRoot := repositoryRootDir(currentDir); projectRoot != "" {
 		return projectRoot, nil
 	}
 
 	return "", errors.WithStack(ErrProjectRootNotFoundNoGitRemote)
 }
 
-func potentialCurrentEnvironmentID(cwd string) (string, error) {
-	for _, potentialEnvironment := range guessCloudBranch(cwd) {
-		return potentialEnvironment, nil
-	}
-
-	return "", errors.New("no known git upstream, branch or environment name")
-}
-
 func repositoryRootDir(currentDir string) string {
 	for {
 		f, err := os.Stat(filepath.Join(currentDir, ".git"))
@@ -99,23 +96,21 @@ func repositoryRootDir(currentDir string) string {
 	return ""
 }
 
-func guessProjectRoot(currentDir string, debug bool) (string, string) {
-	rootDir := repositoryRootDir(currentDir)
-	if rootDir == "" {
-		return "", ""
+func getProjectID(projectRoot string, debug bool) string {
+	brand := GuessCloudFromDirectory(projectRoot)
+	id := getProjectIDFromConfigFile(brand, projectRoot, debug)
+	if id != "" {
+		return id
 	}
-	config := getProjectConfig(rootDir, debug)
-	if config == "" {
-		return "", ""
-	}
-	return rootDir, config
+
+	return getProjectIDFromGitConfig(brand, projectRoot, debug)
 }
 
-func getProjectConfig(projectRoot string, debug bool) string {
-	contents, err := os.ReadFile(filepath.Join(projectRoot, ".platform", "local", "project.yaml"))
+func getProjectIDFromConfigFile(brand CloudBrand, projectRoot string, debug bool) string {
+	contents, err := os.ReadFile(filepath.Join(projectRoot, brand.ProjectConfigPath, "local", "project.yaml"))
 	if err != nil {
 		if debug {
-			fmt.Fprintf(os.Stderr, "WARNING: unable to find Platform.sh config file: %s\n", err)
+			fmt.Fprintf(os.Stderr, "WARNING: unable to find %s config file: %s\n", brand, err)
 		}
 		return ""
 	}
@@ -124,25 +119,24 @@ func getProjectConfig(projectRoot string, debug bool) string {
 	}
 	if err := yaml.Unmarshal(contents, &config); err != nil {
 		if debug {
-			fmt.Fprintf(os.Stderr, "ERROR: unable to decode Platform.sh config file: %s\n", err)
+			fmt.Fprintf(os.Stderr, "ERROR: unable to decode %s config file: %s\n", brand, err)
 		}
 		return ""
 	}
 	return config.ID
 }
 
-func guessCloudBranch(cwd string) []string {
-	localBranch := git.GetCurrentBranch(cwd)
-	if localBranch == "" {
-		return []string{}
+func getProjectIDFromGitConfig(brand CloudBrand, projectRoot string, debug bool) string {
+	for _, remote := range []string{brand.GitRemoteName, "origin"} {
+		url := git.GetRemoteURL(projectRoot, remote)
+		matches := regexp.MustCompile(`^([a-z0-9]{12,})@git\.`).FindStringSubmatch(url)
+		if len(matches) > 1 {
+			return string(matches[1])
+		}
+		return ""
 	}
-
-	branches := []string{}
-	branches = append(branches, localBranch)
-
-	if remoteBranch := git.GetUpstreamBranch(cwd, "origin", "upstream"); remoteBranch != "" {
-		branches = append(branches, remoteBranch)
+	if debug {
+		fmt.Fprintf(os.Stderr, "ERROR: unable to read the git config file\n")
 	}
-
-	return branches
+	return ""
 }
